#!/usr/bin/perl
use warnings;
use strict;
use Config::Simple;
use Net::Jabber qw(Client);
use File::Basename;
use Data::Dumper;

# config file syntax: "option value" 
# options are: server, port, username, password, ressource
# location: ~/.jabberwallrc

if(@ARGV != 1) {
  print "usage: ", basename($0), " <userlist>\n";
  exit 1;
}

open FH, "<", $ARGV[0] or die "$ARGV[0]: $!";
my @recipients = <FH>;
close FH;

my %config;
Config::Simple->import_from("$ENV{HOME}/.jabberwallrc", \%config);

my $clnt = new Net::Jabber::Client;

my $status = $clnt->Connect(hostname=>$config{server}, port=>$config{port});

if (!defined($status)) {
  die "Jabber connect error ($!)\n";
}

my @result = $clnt->AuthSend(username=>$config{username},
  password=>$config{password},
  resource=>$config{resource});

if ($result[0] ne "ok") {
  die "Jabber auth error: @result\n";
}

my $body = '';
while (<STDIN>) {
  $body .= $_;
}
chomp($body);

for my $to (@recipients) {
  chomp $to;
  next if ($to eq "");
  
  $clnt->MessageSend(to=>$to,
    subject=>"",
    body=>$body,
    type=>"chat",
    priority=>0);
}

$clnt->Disconnect();
